        /*** Copyright (C) 1990 Beaver Technology. All rights reserved ***/

#include "stdio.h"
#include "conio.h"
#include "time.h"
#include "graph.h"
#include "sdm.h"                /* header files for SDM */
#include "ti_sup.h"             /* Timer Interrupt support if needed */
#include "sdm_sup.h"            /* SDM utility functions if needed */

int video_mode;                 /* Current video mode */
int vid_error;                  /* Error code if hardware doesn't support mode */

struct menu_def_tmp vdemo_def;  /* declaration of menu which drives video */

int vdemo(int);                 /* some function declarations for this demo */
int d_line(int, int, int, int);
void elapsed_time(int, int);

        /***** global functions for timers and keys ******/
/** The format of global and context function lists is
    similar to printf() format strings except that
    the argument list can include functions to call and their arguments.
    When key code is read in the function list is called.
    Each line of the global function list below has the format
         key code, format string, arguments

    TLEVELO is a key scan code not allocated to the keyboard itself, and
    set each second by one of the interrupt clocks running at 1.1kHz.
    Interrupt clock support at 1.1kHz is provided in the SDM package.
**/

struct func_list_tmp global_list[] =
{
  {TLEVEL0 , "%$", date_time},                  /* refresh date and time */
  {TLEVEL0 , "%$II", elapsed_time, 0, 0},       /* refresh elapsed time */
  {PF1     , "%$II", elapsed_time, 0, 1},       /* reset elapsed time on PF1 */
};
struct func_def_tmp global_def =
{
  global_list, 3
};

                    /*** Header menu ***/
/** The format of menus is similar to printf() format strings except that
    the argument list can include functions to call and their arguments.
    Each line of the menu list below has the format
         key code, format string, arguments

    The macros (eg RONB for Red on Blue) override the default
    menu colors. Any valid attribute byte is valid for text and
    graphics mode. An added feature is that the background color in
    graphics mode can vary over the screen.
**/

struct menu_item_tmp header[] =
{
  { YONB"     "BWONB"Software Development Module "},
  {LMONB"     "BWONB"Computer Assisted Software Engineering"},
  {LRONB"     "},
  {LCONB"     "BWONB"Beaver Technology"},
  {LGONB"     "BWONB"55 Lakeside Drive"},
  {DGONB"     "BWONB"Katonah, New York"},
  { WONB"     "BWONB"(914) 232-8932"},
  {BRONB"      "},
  { MONB"     "BWONB"(C) 1990 Beaver Technology, All Rights Reserved."},
  { RONB"      "},
  { CONB"      "},
  { GONB"      "},
  {" "},
  {" "},
  {LRONB"               Any key to continue"}
};
struct menu_def_tmp header_def =
{
  header,               /* structure above with each menu row. */
  {6, 6, 15},           /* location on the screen row, col; number of rows. */
  {NOBORDER,"Software Development Module", COLOR(BWHITE, BLUE)}, /* border */
  { COLOR(BWHITE, BLUE)},                                       /* default color */
};

     /*** Menu displays prompt at bottom of screen to reset elapsed time ***/
struct menu_item_tmp gaction_list[] =
{
  {LRONB"PF1 - Reset elapsed time"}     /* LRONB - color to light red on blue */
};
struct menu_def_tmp gaction_def =
{
  gaction_list,          /* structure above with each menu row. */
  {24, 10, 1},           /* location on the screen row, col; number of rows. */
  {NOBORDER},                           /* no border or title */
};

struct menu_item_tmp main_list[] =
{
  {YONB"Welcome to the Software Development Module (SDM). "},
  {" "},
  {"   This demo was produced from the Software Development Module. "},
  {" SDM generates programs in text or graphics modes."},
  {" Source input to SDM is identical "},
  {" for text or graphics displays."},
  {" "},
  {YONB"Graphics Mode Demonstration "},
  {"  "},
  {" Present Mode is Text 80X25. "},
  {" Select Video Mode and Press Enter. "},
  {" EGA 16 COLOR (640X200) %$I %^   VGA 16 COLOR (640X480) %$I",
      vdemo, _HRES16COLOR, vdemo, _VRES16COLOR },
  {" Error flag from video (0-unsupported) %d", &vid_error},
};
struct menu_def_tmp main_def =
{
  main_list,
  {5, 10, 13},
  {2," Information ", COLOR(LRED, BLUE)},
  {COLOR(BWHITE,BLUE), COLOR(RED,BLUE), COLOR(BLUE, WHITE), COLOR(RED ,WHITE)},
};

main()
{
  clear_screen(COLOR(WHITE, BLUE));

           /*** display header menu ***/
  call_menu(&header_def, WRITE, NOERASE);
  while(!kbhit); getch();               /* any key to continue */
  call_menu(&header_def, WRITE, ERASE);

           /**** Turn on interrupt clock 0 to update timers *****/
  init_intr( 0, (long) 1000, TLEVEL0);  /* clock 0, every 1000 ms, to kb_buf */
  intr_on();            /* timer interrupts on */

           /**** global functions on *****/
  set_gfuncs(&global_def, 1);
  call_menu(&gaction_def, WRITE, NOERASE);      /* PF1 to reset elapsed time */
  call_menu(&main_def, NORMAL, NOERASE);        /* main menu */

  intr_off();                                   /**** interrupts off *****/
  clear_screen(COLOR(BWHITE, GREEN));
}       /* end main */

int c_window(void)              /* clears graphics clip area */
{
  _setcolor(WHITE);
  _floodfill(260,100, BLUE);
  _setcolor(RED);
}

int xul=260, yul=60, xlr=500, ylr=270;
int line_col=YELLOW;

       /**** script menu of main graphics menu gscreen_def *****/
       /**** put on screen to show SDM input for menu  ****/
struct menu_item_tmp dscreen_list[] =
{
  {"YONB Graphics Parameters"},
  {"                  X    Y"},
  {"  Upper left;   %%?3d  %%^ %%?3d\", &xul, &yul"},
  {"  Lower right;  %%?3d  %%^ %%?3d\", &xlr, &ylr"},
  {"YONB Line color "},
  {"     YELLOW %%$I  %%^\"YONW\"\", _setcolor, YELLOW"},
  {"     RED    %%$I  %%^\"RONW\"\", _setcolor, RED"},
  {"     GREEN  %%$I  %%^\"GONW\"\", _setcolor, GREEN"},
  {"YONB Draw Shape"},
  {"  LINE %%$dddd\", d_line, &xul,&yul,&xlr,&ylr"},
  {"  RECTANGLE %%$Idddd\", _rectangle, _GBORDER, &xul,&yul,&xlr,&ylr"},
  {"  ELLIPSE %%$Idddd\", _ellipse, _GBORDER, &xul,&yul,&xlr,&ylr"},
  {"YONB"" Clear View Port %%$\",c_window"},
};
struct menu_def_tmp dscreen_def =
{
  dscreen_list,
  {9, 6, 13},
  {DLINE," SDM input for Demonstration ", COLOR(LRED, BLUE)},
  {COLOR(BWHITE,BLUE)}
};

int c_menu()            /** needed to restore screen under menu **/
{                       /** in graphics mode due to bug in Microsoft **/
  switch(video_mode)    /** put_image()/get_image() routines **/
  {
    case _HRES16COLOR:
     _setcliprgn(0,0,640,480);
     call_menu( &dscreen_def, NORMAL | SAVE, ERASE | RESTORE);
     _setcliprgn(250, 25, 630, 180);
    break;

    case _VRES16COLOR:
     _setcliprgn(0,0,640,480);
     call_menu( &dscreen_def, NORMAL | SAVE, ERASE | RESTORE);
     _setcliprgn(230, 40, 630, 440);
    break;
  }
}

       /**** Main graphics menu, note direct function calls  *****/
       /**** from data structure to d_line etc. with arg passing ****/
struct menu_item_tmp gscreen_list[] =                   /* graphics menu */
{
  {YONB" Graphics Positions"},
  {" "},
  {"                  X    Y"},
  {"  Upper left;%^   %?3d  %^ %?3d", &xul, &yul},
  {"  Lower right;%^  %?3d  %^ %?3d", &xlr, &ylr},
  {" "},
  {YONB" Line color "},
  {"     YELLOW %$I  %^"YONW"", _setcolor, YELLOW},
  {"     RED    %$I  %^"RONW"", _setcolor, RED},
  {"     GREEN  %$I  %^"GONW"", _setcolor, GREEN},
  {" "},
  {YONB" Draw Shape"},
  {"  LINE %$dddd", d_line, &xul,&yul,&xlr,&ylr},
  {"  RECTANGLE %$Idddd", _rectangle, _GBORDER, &xul,&yul,&xlr,&ylr},
  {"  ELLIPSE %$Idddd", _ellipse, _GBORDER, &xul,&yul,&xlr,&ylr},
  {" "},
  {YONB" Clear View Port %$",c_window},
  {YONB" SDM input for program %$",c_menu},
};
struct menu_def_tmp gscreen_def =
{
  gscreen_list,
  {4, 1, 18},
  {DLINE," Information ", COLOR(LRED, BLUE)},
  {COLOR(BWHITE,BLUE), COLOR(RED,BLUE), COLOR(RED, WHITE), COLOR(RED ,WHITE)},
};

int vdemo(int mode)             /* main graphics routine */
{
  video_mode = mode;
  if((vid_error=_setvideomode(mode)) == 0) return;      /* set video mode */
  _setbkcolor(_BLUE);

  switch(mode)                  /* set up clip region on graphics screen */
  {
    case _HRES16COLOR:
      _settextcolor(WHITE);
      _setcliprgn(250, 25, 630, 180);
      _settextposition(3, 30);  /* label coordinates of clip region */
      _outtext("(250, 25)");
      _settextposition(24, 71);
      _outtext("(630, 180)");
    break;

    case _VRES16COLOR:
      _settextcolor(WHITE);
      _settextposition(2, 23);
      _outtext("(230, 40)");
      _settextposition(29, 71);
      _outtext("(630, 440)");
     _setcliprgn(230, 40, 630, 440);
    break;
  };

  _setcolor(WHITE);
  _floodfill(260, 100, BLUE);                   /* fill viewport WHITE */
  _setcolor(RED);
  call_menu( &gscreen_def, NORMAL, NOERASE);
  _setvideomode(_DEFAULTMODE);                  /* restore text mode */
  clear_screen(COLOR( WHITE,BLUE));
  call_menu(&gaction_def, WRITE, NOERASE);      /* restore PF1 - reset menu */
}


      /***** utility functions ****/

int d_line(int xul, int yul, int xlr, int ylf)  /* line from A to B */
{
  _moveto( xul, yul);
  _lineto( xlr, ylr);
}

void elapsed_time(int timer, int mode)  /* timer 0-NTIMERS, mode 1-init 0-rd */
{
  static int hrs, mins, secs;
  long i;

  static struct menu_item_tmp time_list[] =
  {
    { "Elapsed Time %.2d:%.2d:%.2d", &hrs, &mins, &secs},
  };
  static struct menu_def_tmp timer_def =
  {
    time_list,
    { 0, 30, 1},
    { NOBORDER, "elapsed_time", COLOR(YELLOW, BLUE)},
    { COLOR(LRED, BLUE),}
  };

    i = interval(timer, (long) mode)/1164;     /* interval in secs */
    hrs  = i/3600;
    mins = (i%3600)/60;
    secs = i%60;
    call_menu(&timer_def, WRITE, NOERASE);
}

